import maya.OpenMayaMPx as OpenMayaMPx
import maya.OpenMaya    as OpenMaya
import maya.OpenMayaRender as OpenMayaRender
import maya.OpenMayaUI as OpenMayaUI
import math

import sys

kPluginNodeTypeName = 'lightReflRay'
kPluginNodeId = OpenMaya.MTypeId( 0x00102ECC )
kPluginClassify = "utility/light"

glRenderer = OpenMayaRender.MHardwareRenderer.theRenderer()
glFT = glRenderer.glFunctionTable()


DEBUG = False

class lightReflRay( OpenMayaMPx.MPxLocatorNode ):


    def __init__( self ):
        OpenMayaMPx.MPxLocatorNode.__init__( self )
        # attributes

        # incidence point
        aPointI     =  OpenMaya.MObject()
        # components of I
        aPointIX    =  OpenMaya.MObject()
        aPointIY    =  OpenMaya.MObject()
        aPointIZ    =  OpenMaya.MObject()
        # normal vector
        aVecN     =  OpenMaya.MObject()
        # components of N. Type: float
        aVecNX    =  OpenMaya.MObject()
        aVecNY    =  OpenMaya.MObject()
        aVecNZ    =  OpenMaya.MObject()
        # reflecting target point
        aPointT   =  OpenMaya.MObject()
        # components of S. Type: float
        aVecTX    =  OpenMaya.MObject()
        aVecTY    =  OpenMaya.MObject()
        aVecTZ    =  OpenMaya.MObject()
        # reflection point R. Type: float
        aPointR   =  OpenMaya.MObject()
        # components or R. Type: float
        aPointRX  =  OpenMaya.MObject()
        aPointRY  =  OpenMaya.MObject()
        aPointRZ  =  OpenMaya.MObject()
        # length of the reflection line. Type: float
        aDistance =  OpenMaya.MObject()
        # radius of the drawing circle/rectangle. Type: float
        aRadius   =  OpenMaya.MObject()
        # display the reflection line or not. Type: bool
        aDisplay  =  OpenMaya.MObject()
        # normal circle points array
        aNCPArray =  OpenMaya.MObject()
        # reflection rectangle points array
        aRRPArray =  OpenMaya.MObject()


    def compute( self, plug, data ):

        thisNode = self.thisMObject()
        if plug == lightReflRay.aPointR \
            or plug ==lightReflRay.aPointRX \
            or plug ==lightReflRay.aPointRY \
            or plug ==lightReflRay.aPointRZ \
             :

            pointI = data.inputValue(lightReflRay.aPointI).asVector()
            pointT = data.inputValue(lightReflRay.aPointT).asVector()
            vecN   = data.inputValue(lightReflRay.aVecN).asVector()
            dist   = data.inputValue(lightReflRay.aDistance).asFloat()
            radius = data.inputValue(lightReflRay.aRadius).asFloat()

            vectorI = pointT - pointI # mvector
            vectorI.normalize()
            vectorN = OpenMaya.MVector(vecN) # mvector
            vectorN.normalize()

            dotP    = vectorI*vectorN
            vectorNN= OpenMaya.MVector(2*dotP*vectorN.x, 2*dotP*vectorN.y, 2*dotP*vectorN.z )           
            vectorR = vectorI - vectorNN
            vectorR.normalize()  # normalize it

            pointX  = pointT.x + vectorR.x*dist 
            pointY  = pointT.y + vectorR.y*dist 
            pointZ  = pointT.z + vectorR.z*dist 
            pointR  = OpenMaya.MVector(pointX, pointY, pointZ)

            
            nn_rotMtx = self.getVectorTransformMatrix(vecN)
            # store the normal cirle points
            normalPointsArray = OpenMaya.MVectorArray()
            cirPointsNum = 16
            pointsList = self.getCirclePointsXZ(radius,cirPointsNum )
            for point in pointsList:
                nv = point*nn_rotMtx + pointT
                normalPointsArray.append(nv)


            nr_rotMtx = self.getVectorTransformMatrix(vectorR)
            # store the reflection draw points
            reflRecPointsArray = OpenMaya.MVectorArray()
            recPointsNum = 4
            pointsList = self.getCirclePointsXZ(radius,recPointsNum )
            for point in pointsList:
                nv = point*nr_rotMtx + pointR
                reflRecPointsArray.append(nv)

            if DEBUG:
                print "reflection point values are:", pointX, pointY, pointZ

            dataHandle = data.outputValue(lightReflRay.aPointR)
            dataHandle.setMVector(pointR)
            dataHandle.setClean()


            vectorArrayDataFn = OpenMaya.MFnVectorArrayData()
            dataHandle = data.outputValue(lightReflRay.aNCPArray)
            dataHandle.setMObject( vectorArrayDataFn.create( normalPointsArray) )
            dataHandle.setClean()

            vectorArrayDataFn = OpenMaya.MFnVectorArrayData()
            dataHandle = data.outputValue(lightReflRay.aRRPArray)
            dataHandle.setMObject( vectorArrayDataFn.create( reflRecPointsArray) )
            dataHandle.setClean()
            
        else:
            #return unkown

            #
            # since Maya2011 the MStatus has been removed in python API
            #
            try:
                return OpenMaya.MStatus.kUnknownParameter
            except:
                raise RuntimeError('OpenMaya.MStatus.kUnknownParameter')

        try:
            return OpenMaya.MStatus.kSuccess
        except:
            return

    def getVectorTransformMatrix(self, vec):
        # get an arbitrary tangent vector of vector
        NN = vec^OpenMaya.MVector(0,1,0)
        x = vec^NN
        x.normalize()
        vec.normalize()
        NN.normalize()

        matrixList = [ x.x, x.y, x.z, 0, \
                       vec.x, vec.y, vec.z, 0, \
                       NN.x, NN.y, NN.z, 0, \
                       0, 0, 0, 1 ]

        transformMtx = TransformAPI().getTransformMatrixFromList(matrixList)
        rotMtx = transformMtx.eulerRotation().asMatrix()
        
        return rotMtx

    def drawCircleXZ(self, pointsArray,  color):
        
        glFT.glBegin(OpenMayaRender.MGL_LINE_LOOP)
        glFT.glColor3f(color[0], color[1],color[2])
        for i in xrange(pointsArray.length()):
            v = pointsArray[i]
            glFT.glVertex3f(v.x, v.y, v.z)
        glFT.glEnd()
        
    def getCirclePointsXZ(self, radius, pointCount):
        pointsList = []
        radius = abs(radius)
        i = 0
        while i < pointCount:
            angle = float(i*(2*math.pi)/pointCount)
            ca = math.cos(angle)
            sa = math.sin(angle)
            y  = 0
            x  = ca*radius
            z  = sa*radius

            vP = OpenMaya.MVector(x, y, z)
            pointsList.append(vP)
            i +=1

        return pointsList


    def draw(self, view, path, style, status):
        thisNode = self.thisMObject()

        # get display option
        display = OpenMaya.MPlug(thisNode, self.aDisplay).asBool()
        if not display:
            return

        # get Incidence point
        pointIX = OpenMaya.MPlug(thisNode, self.aPointIX).asFloat()
        pointIY = OpenMaya.MPlug(thisNode, self.aPointIY).asFloat()
        pointIZ = OpenMaya.MPlug(thisNode, self.aPointIZ).asFloat()
       # get Reflecting Point T
        pointTX = OpenMaya.MPlug(thisNode, self.aPointTX).asFloat()
        pointTY = OpenMaya.MPlug(thisNode, self.aPointTY).asFloat()
        pointTZ = OpenMaya.MPlug(thisNode, self.aPointTZ).asFloat()
        # get Reflection Point R
        pointRX = OpenMaya.MPlug(thisNode, self.aPointRX).asFloat()
        pointRY = OpenMaya.MPlug(thisNode, self.aPointRY).asFloat()
        pointRZ = OpenMaya.MPlug(thisNode, self.aPointRZ).asFloat()
        # get Normal N
        pointNX = OpenMaya.MPlug(thisNode, self.aVecNX).asFloat()
        pointNY = OpenMaya.MPlug(thisNode, self.aVecNY).asFloat()
        pointNZ = OpenMaya.MPlug(thisNode, self.aVecNZ).asFloat()

        
        # normal cicle points array
        oData  =  OpenMaya.MPlug(thisNode, self.aNCPArray).asMObject()
        fnData =  OpenMaya.MFnVectorArrayData(oData) 
        ncpArray = fnData.array()

        # rec point array 
        oData  =  OpenMaya.MPlug(thisNode, self.aRRPArray).asMObject()
        fnData =  OpenMaya.MFnVectorArrayData(oData) 
        rrpArray = fnData.array()
        

        if DEBUG:
            print "pointIX is", pointIX
            print "pointIY is", pointIY
            print "pointIZ is", pointIZ

            print "pointTX is", pointTX
            print "pointTY is", pointTY
            print "pointTZ is", pointTZ

            print "pointRX is", pointRX
            print "pointRY is", pointRY
            print "pointRZ is", pointRZ

        # I ray color
        iRayColor = [0.1,0.60,0.7]
        # R ray color
        rRayColor = [1.0,0.35,0.0]
        # reflection circle color
        cicleColor= [0.2,0.425,0.35]
        # rectangle color
        rectColor = [0.112,0.5,0.75]

        #glFT.glEnable(OpenMayaRender.MGL_BLEND)
        #glFT.glEnable(OpenMayaRender.MGL_LINE_SMOOTH)
        #glFT.glHint(OpenMayaRender.MGL_LINE_SMOOTH_HINT,OpenMayaRender.MGL_NICEST)

        view.beginGL()
        glFT.glPushAttrib(OpenMayaRender.MGL_CURRENT_BIT)

        # draw the I ray
        glFT.glBegin(OpenMayaRender.MGL_LINES)
        glFT.glColor3f(iRayColor[0], iRayColor[1],iRayColor[2])
        glFT.glVertex3f(pointIX,pointIY,pointIZ )
        glFT.glVertex3f(pointTX,pointTY,pointTZ )
        glFT.glEnd()

        
        # draw the Reflection point circle
        self.drawCircleXZ(ncpArray,  cicleColor)
        
        # draw to R ray
        glFT.glBegin(OpenMayaRender.MGL_LINES)
        glFT.glColor3f(rRayColor[0], rRayColor[1],rRayColor[2])
        glFT.glVertex3f(pointTX,pointTY,pointTZ )
        glFT.glVertex3f(pointRX,pointRY,pointRZ )
        glFT.glEnd()

        
        # draw the ray rectangle 
        self.drawCircleXZ(rrpArray,  rectColor)

        if rrpArray:
            if rrpArray.length():
                glFT.glBegin(OpenMayaRender.MGL_LINES)
                glFT.glColor3f(rectColor[0], rectColor[1],rectColor[2])
                v0 = rrpArray[0]
                v1 = rrpArray[1]
                v2 = rrpArray[2]
                v3 = rrpArray[3]

                glFT.glVertex3f(v0.x,v0.y,v0.z )
                glFT.glVertex3f(v2.x,v2.y,v2.z )
                glFT.glVertex3f(v1.x,v1.y,v1.z )
                glFT.glVertex3f(v3.x,v3.y,v3.z )
                glFT.glEnd()
        
        
        glFT.glPopAttrib()
        view.endGL()


def nodeCreator():
    return OpenMayaMPx.asMPxPtr( lightReflRay() )


def nodeInitializer():

    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aDisplay = nAttr.create( "displayRays", "dr",OpenMaya.MFnNumericData.kBoolean , True )
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aDisplay )

    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aDistance = nAttr.create( "rayDistance", "rd",OpenMaya.MFnNumericData.kFloat , 2.0 )
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aDistance )

    # drawing circle radius 
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aRadius = nAttr.create( "drawingRadius", "cdr",OpenMaya.MFnNumericData.kFloat  , 1.0 )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    nAttr.setMin(0.1)
    #nAttr.setHidden(True)
    lightReflRay.addAttribute( lightReflRay.aRadius )

    # Ix
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointIX = nAttr.create( "eyePointX", "epx" , OpenMaya.MFnNumericData.kDouble, 1.0)    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointIX )

    # Iy
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointIY = nAttr.create( "eyePointY", "epy" ,OpenMaya.MFnNumericData.kDouble , 1.0)    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointIY )

    # Iz
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointIZ = nAttr.create( "eyePointZ", "epz" , OpenMaya.MFnNumericData.kDouble, 1.0 )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointIZ )

    # I
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointI = nAttr.create( "eyePoint", "ep" ,lightReflRay.aPointIX, lightReflRay.aPointIY, lightReflRay.aPointIZ )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointI )

    # Tx
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointTX = nAttr.create( "reflTargetPointX", "rtpx",OpenMaya.MFnNumericData.kDouble , 0.0 )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointTX )

    # Ty
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointTY = nAttr.create( "reflTargetPointY", "rtpy",OpenMaya.MFnNumericData.kDouble , 0.0 )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointTY )

    # Tz
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointTZ = nAttr.create( "reflTargetPointZ", "rtpz", OpenMaya.MFnNumericData.kDouble, 0.0 )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointTZ )

    # T
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointT = nAttr.create( "reflTargetPoint", "rtp", lightReflRay.aPointTX, lightReflRay.aPointTY, lightReflRay.aPointTZ )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointT )


    # Rx
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointRX = nAttr.create( "reflectionPointX", "rpx",OpenMaya.MFnNumericData.kDouble , 1.0 )    
    nAttr.setStorable(True)
    nAttr.setHidden(True)
    #nAttr.setKeyable(True)
    #nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointRX )

    # Ry
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointRY = nAttr.create( "reflectionPointY", "rpy",OpenMaya.MFnNumericData.kDouble , -1.0)    
    nAttr.setStorable(True)
    nAttr.setHidden(True)
    #nAttr.setKeyable(True)
    #nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointRY )

    # Rz
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointRZ = nAttr.create( "reflectionPointZ", "rpz",OpenMaya.MFnNumericData.kDouble , 1.0 )    
    nAttr.setStorable(True)
    nAttr.setHidden(True)
    #nAttr.setKeyable(True)
    #nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointRZ )

    # R
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aPointR = nAttr.create( "reflectionPoint", "rp", lightReflRay.aPointRX , lightReflRay.aPointRY , lightReflRay.aPointRZ  )    
    nAttr.setStorable(True)
    nAttr.setHidden(True)
    #nAttr.setKeyable(True)
    #nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aPointR )

    # Nx
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aVecNX = nAttr.create( "reflectionNormalX", "rnx",OpenMaya.MFnNumericData.kDouble  , 1.0 )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aVecNX )

    # Ny
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aVecNY = nAttr.create( "reflectionNormalY", "rny",OpenMaya.MFnNumericData.kDouble , 1.0 )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aVecNY )

    # Nz
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aVecNZ = nAttr.create( "reflectionNormalZ", "rnz",OpenMaya.MFnNumericData.kDouble  , 1.0 )    
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aVecNZ )

    # N
    nAttr = OpenMaya.MFnNumericAttribute()
    lightReflRay.aVecN = nAttr.create( "reflectionNormal", "rn", lightReflRay.aVecNX, lightReflRay.aVecNY, lightReflRay.aVecNZ  )
    nAttr.setStorable(True)
    nAttr.setKeyable(True)
    nAttr.setConnectable(True)
    lightReflRay.addAttribute( lightReflRay.aVecN )

    # normal circle points array
    tAttr = OpenMaya.MFnTypedAttribute()
    defaultVectorArray = OpenMaya.MVectorArray()
    vectorArrayDataFn = OpenMaya.MFnVectorArrayData()
    vectorArrayDataFn.create( defaultVectorArray )
    lightReflRay.aNCPArray = tAttr.create( "normalCirclePointsAttry", "ncpa",OpenMaya.MFnData.kVectorArray, vectorArrayDataFn.object())
    tAttr.setKeyable(False)
    tAttr.setConnectable(False)
    tAttr.setStorable(False)
    tAttr.setHidden(True)
    lightReflRay.addAttribute( lightReflRay.aNCPArray )

    # reflection rectangle points array
    tAttr = OpenMaya.MFnTypedAttribute()
    defaultVectorArray = OpenMaya.MVectorArray()
    vectorArrayDataFn = OpenMaya.MFnVectorArrayData()
    vectorArrayDataFn.create( defaultVectorArray )
    lightReflRay.aRRPArray = tAttr.create( "reflectionRectanglePointsAttry", "nrpa",OpenMaya.MFnData.kVectorArray,vectorArrayDataFn.object() )
    tAttr.setKeyable(False)
    tAttr.setConnectable(False)
    tAttr.setStorable(False)
    tAttr.setHidden(True)
    lightReflRay.addAttribute( lightReflRay.aRRPArray )

    
    # relationships
    lightReflRay.attributeAffects( lightReflRay.aDistance, lightReflRay.aPointR )

    lightReflRay.attributeAffects( lightReflRay.aPointIX,  lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aPointIY,  lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aPointIZ,  lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aPointI,   lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aPointI,   lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aPointI,   lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aPointI,   lightReflRay.aPointRZ )

    lightReflRay.attributeAffects( lightReflRay.aPointIX,   lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aPointIX,   lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aPointIX,   lightReflRay.aPointRZ )
    lightReflRay.attributeAffects( lightReflRay.aPointIY,   lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aPointIY,   lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aPointIY,   lightReflRay.aPointRZ )
    lightReflRay.attributeAffects( lightReflRay.aPointIZ,   lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aPointIZ,   lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aPointIZ,   lightReflRay.aPointRZ )


    lightReflRay.attributeAffects( lightReflRay.aPointTX,  lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aPointTY,  lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aPointTZ,  lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aPointT,   lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aPointT,   lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aPointT,   lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aPointT,   lightReflRay.aPointRZ )

    lightReflRay.attributeAffects( lightReflRay.aPointTX,   lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aPointTX,   lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aPointTX,   lightReflRay.aPointRZ )
    lightReflRay.attributeAffects( lightReflRay.aPointTY,   lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aPointTY,   lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aPointTY,   lightReflRay.aPointRZ )
    lightReflRay.attributeAffects( lightReflRay.aPointTZ,   lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aPointTZ,   lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aPointTZ,   lightReflRay.aPointRZ )


    lightReflRay.attributeAffects( lightReflRay.aVecNX,    lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aVecNY,    lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aVecNZ,    lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aVecN,     lightReflRay.aPointR )
    lightReflRay.attributeAffects( lightReflRay.aVecN,     lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aVecN,     lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aVecN,     lightReflRay.aPointRZ )

    lightReflRay.attributeAffects( lightReflRay.aVecNX,     lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aVecNX,     lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aVecNX,     lightReflRay.aPointRZ )
    lightReflRay.attributeAffects( lightReflRay.aVecNY,     lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aVecNY,     lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aVecNY,     lightReflRay.aPointRZ )
    lightReflRay.attributeAffects( lightReflRay.aVecNZ,     lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aVecNZ,     lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aVecNZ,     lightReflRay.aPointRZ )


    lightReflRay.attributeAffects( lightReflRay.aRadius,    lightReflRay.aPointR )

    lightReflRay.attributeAffects( lightReflRay.aRadius,     lightReflRay.aPointRX )
    lightReflRay.attributeAffects( lightReflRay.aRadius,     lightReflRay.aPointRY )
    lightReflRay.attributeAffects( lightReflRay.aRadius,     lightReflRay.aPointRZ )



# initialize the script plug-in    
def initializePlugin( mobject ):
    mplugin = OpenMayaMPx.MFnPlugin( mobject, "DD", "1.0", "Any" )
    try:
        mplugin.registerNode( kPluginNodeTypeName, kPluginNodeId, nodeCreator, nodeInitializer,
                              OpenMayaMPx.MPxNode.kLocatorNode, kPluginClassify )
    except:
        sys.stderr.write( "Failed to register node: %s" % kPluginNodeTypeName )
        raise

# uninitialize the script plug-in
def uninitializePlugin(mobject):
    mplugin = OpenMayaMPx.MFnPlugin(mobject)
    try:
        mplugin.deregisterNode( kPluginNodeId )
    except:
        sys.stderr.write( "Failed to deregister node: %s" % kPluginNodeTypeName )
        raise










######################################################################
# api helper
# an maya python api helper, for better(maybe) access of 
# some common maya scripting task
# lli@d2.com, howandwhen@gmail.com
#########################################################


import maya.cmds as cmds
import math
import time
import math,os,glob,re




class BasicAPI:
    """ Basic API function class set"""
    def __init__(self):
        self.space = OpenMaya.MSpace.kWorld
        self.MSUtil= OpenMaya.MScriptUtil()
        pass


    def getCurrSel_API(self):
        """ get currenct selections in API style """
        currSelList = OpenMaya.MSelectionList()   
        OpenMaya.MGlobal.getActiveSelectionList(currSelList)
        return currSelList


    def setCurrSel_API(self,currSelList):
        """ set currenct selections in API style """
        if isinstance(currSelList, OpenMaya.MSelectionList) :
            OpenMaya.MGlobal.setActiveSelectionList(currSelList,OpenMaya.MGlobal.kReplaceList)
        # end if isinstance
        else:
            print ("%s is not a selection List. Exiting! \n") % currSelList
        # end else


    def isValidMDagNode (self,obj):
        """ check input MObject has kDagNode function set """
        if self.isValidMObject(obj) :
            return obj.hasFn(OpenMaya.MFn.kDagNode)
        # end if
        else :
            return False
        # end else
    
    def isValidMObject (self,obj):
        """ check if it's a MObject instance """
        if isinstance(obj, OpenMaya.MObject) :
            return not obj.isNull()
        # end if
        else :
            return False 

    def vectorXYZPrint(self,MVector):
        """print the x y z value of a MVector"""
        print "Mvector x,y,z values are:\n %s, %s, %s\n" %(MVector.x, MVector.y, MVector.z)

    def pointsToVector(self, MPoint1, MPoint2):
        """ Return a MVector from two MPoints"""
        MVector = MPoint1 - MPoint2
        return MVector
    
    def nameTo_Dag_Mobject( self,name,sel):
        """ get dag node and mobject from string names """
        sel.clear()
        sel.add(name)
        path = OpenMaya.MDagPath()
        obj = OpenMaya.MObject()
        sel.getDagPath(0, path)
        sel.getDependNode(0,obj)
        return path,obj

    def nameToDag(self, name, sel):
        sel.clear()
        sel.add(name)
        mDagPath = OpenMaya.MDagPath()
        sel.getDagPath(0, mDagPath)
        return mDagPath

    def getComponentNames(self,dagPath,MObject,sel):
        """ get component string names. Input MObject must be a component MObject"""
        sel.clear()
        sel.add(dagPath, MObject)
        compNames = []
        sel.getSelectionStrings(0, compNames)
        # Notice: output is a string array
        return compNames 

    def angleBetween(self, MVector1, MVector2):
        """ get the angel bewteen vector1 and vector 2"""
        return MVector1.angle(MVector2)

    def normalizeMVector(self,MVector):
        """ get normalized MVector"""
        MVector.normalize()
        return MVector

    def getRotationAngles(self, eulerRot):
        """get euler rotaion values in degrees """
        angles =  [math.degrees(angle) for angle in (eulerRot.x, eulerRot.y, eulerRot.z)]
        return angles


class TransformAPI(BasicAPI):
    """ extended class for transform matrix"""
    def __init__(self, name = None):
        BasicAPI.__init__(self)
        self.selList = OpenMaya.MSelectionList()
        self.name = name
        self.rotOrder = OpenMaya.MEulerRotation.kXYZ

    def getTransformMatrix(self, name):
        """get transform Matrix based on transform string name, note:
        this transform matrix is still local transform"""
        mdagPath = self.nameToDag(name, self.selList)
        transformFunc = OpenMaya.MFnTransform(mDagPath) # MFnTransform
        mTransformMtx = transformFunc.transformation() # MTransformationMatrix
        return mTransformMtx

    def getTransformMatrixFromList(self, matrixList):
        """
        get transfomr matrix from a list
		"""
        mMatrix = OpenMaya.MMatrix()
        self.MSUtil.createMatrixFromList(matrixList,mMatrix)
        mTranfomMtx  = OpenMaya.MTransformationMatrix(mMatrix)
        return mTranfomMtx

    def getEulerRotation(self, mtransformMtx):
        eulerRot =  mTransformMtx.eulerRotation() # MEulerRotation
        return eulerRot


    

