import maya.cmds as cmds
import maya.mel as mel
import sys
import traceback
import math


CAM_TYPES = [
    "camera",
    "stereoRigCamera",
]

def warnMessage(msg = "default warning"):
    mel.eval( 'error ":%s" '  % msg )
    return


def checkSelected():
    sels = cmds.ls(sl=1) or []
    if len(sels) != 2:
        return False

    cam = False
    geo = False
    for sel in sels:
        shapes = cmds.listRelatives(sel, s=1) or []
        if shapes:
            shape = shapes[0]
        else:
            shape = sel
        if cmds.objectType(shape) in CAM_TYPES:
            cam = cmds.listRelatives(shape, p=1, pa=1)[0]

        elif  cmds.objectType(shape) == 'mesh' or  cmds.objectType(shape) == 'nurbsSurface':
            geo = cmds.listRelatives(shape, p=1, pa=1)[0]

    if cam and geo:
        return (cam, geo)
    return False


def loadLightReflRayPlugin():
    if not cmds.pluginInfo("lightReflRay", query=True, l=True):
        try:
            mel.eval("warning \"lightReflRay Plugin not loaded. Loading.. \"")
            cmds.loadPlugin("lightReflRay.py")
            
        except:
            mel.eval("warning \"loading lightReflRay Plugin failed. Exiting.. \"")
            print "Error:", sys.exc_info()[0] 
            (tbtype, val, tb) = sys.exc_info()
            traceback.print_exception(tbtype, val, tb)
            return False
    return True
        

def createCamConstraint(cam):
    locName = "%s_parentConstraint_loc" %cam
    loc = cmds.spaceLocator()[0]
    loc = cmds.rename(loc, locName)
    cmds.parentConstraint(cam, loc )
    return loc
    

def createGeoConstrains(geo):
    locName = "%s_constrains_loc" %geo
    loc = cmds.spaceLocator()[0]
    loc = cmds.rename(loc, locName)

    geoCons = cmds.geometryConstraint(geo, loc)
    normalCons = cmds.normalConstraint(geo, loc)
    return loc, normalCons[0]

def createLightLocator(cam, geo):
    locName = "%s_%s_lightPostion_loc" %(cam, geo)
    loc = cmds.spaceLocator()[0]
    loc = cmds.rename(loc, locName)
    print loc
    return loc


def createLightReflRayNode(cam, geo):
    pnode = cmds.createNode('transform', name = "%s_%s_%sTransform" %(cam, geo, 'lightReflRay'))
    name = "%s_%s_%sShape" %(cam, geo, 'lightReflRay')
    node = cmds.createNode('lightReflRay', name=name, parent = pnode)
    cmds.setAttr("%s.rayDistance" %node, 10)
    #print node
    return pnode, node    


def getDistance(obj1, obj2):
    p1 = cmds.xform(obj1,q=1,ws=1,t=1)
    p2 = cmds.xform(obj2,q=1,ws=1,t=1)
    distance = dist(p1,p2)
    return distance

def getBoundingBoxAverage(obj):
    bbx =  cmds.xform(obj,ws=1, q=1, bb=1)
    x   =  abs(bbx[3]-bbx[0])
    y   =  abs(bbx[4]-bbx[1])
    z   =  abs(bbx[5]-bbx[2])
    return 0.618*0.33*(x+y+z)

def setLocScale(loc, radius):
    locs = cmds.listRelatives(loc, shapes=True)
    if locs:
        locShape = locs[0]
        cmds.setAttr("%s.localScaleX" %locShape, radius)
        cmds.setAttr("%s.localScaleY" %locShape, radius)
        cmds.setAttr("%s.localScaleZ" %locShape, radius)

 
def dist(p1,p2):
    distance = math.sqrt( math.pow(p1[0]-p2[0],2) +  math.pow(p1[1]-p2[1],2) + math.pow(p1[2]-p2[2],2) )
    return distance    

def placeHighlight():
    
    selected = checkSelected()
    if not selected:
        warnMessage("Please select one Camera and one Geometry, and try it again.")
        return

    loaded = loadLightReflRayPlugin()
    if not loaded:
        warnMessage("loading Light ray plugin failed. See script editor for details")
        return

    cam = selected[0]
    geo = selected[1]

    camLoc = createCamConstraint(cam)
    geoLoc, normalCons = createGeoConstrains(geo)    
    lightLoc = createLightLocator(cam, geo)

    trans, shape = createLightReflRayNode(cam, geo)

    # connect attrs
    # eye point
    cmds.connectAttr("%s.center" %camLoc, "%s.eyePoint" %shape, f=True)

    # target point
    cmds.connectAttr("%s.center" %geoLoc, "%s.reflTargetPoint" %shape, f=True)

    # normal vector
    cmds.connectAttr("%s.constraintVector" %normalCons, "%s.reflectionNormal" %shape, f=True)

    # out put reflection point
    cmds.connectAttr("%s.reflectionPoint" %shape, "%s.translate" %lightLoc, f=True)

    distance = getDistance(cam, geoLoc)
    # set distance attr 
    cmds.setAttr("%s.rayDistance" %shape, distance)

    # set radius attr, based on geo's bounding box size
    radius = getBoundingBoxAverage(geo)
    cmds.setAttr("%s.drawingRadius" %shape, radius)

    setLocScale(lightLoc, radius)
    setLocScale(geoLoc, radius)

    groupList = [camLoc, geoLoc, lightLoc, trans]
    groupName = "%s_%s_lightRelfRay_Grp" %(cam, geo)

    cmds.group(groupList, name = groupName)

    # select THE locator
    cmds.select(geoLoc, r =1)

    print "done"



